function CardListController($scope, $element, ScreenService, ApplicationContext, MessageService) {

    let htmlElement = $element.get(0);
    let list;
    let isLoading = false;

    let dataSet = [];
    let widget;
    let dataSource;

    let fieldsToDisplayOnCard = [{
        "id": "NMALCADA",
        "label": "Alçada" 
    }, {
        "id": "VRTOTALAF",
        "label": "R$"
    }, {
        "id": "NRDEMANDA",
        "label": "Nº Demanda"
    }, {
        "id": "DTINCLUSAO",
        "label": "Incluído em:"
    }, {
        "id": "NMFILIAL",
        "label": "Unidade" 
    }, {
        "id": "NMFANTFORN",
        "label": "Fornecedor" 
    }, {
        "id": "NMOPERADOR",
        "label": "Solicitante:"
    }, {
        "id": "DSMOTVAFALCADA",
        "label": "Motivo do Lançamento:"
    }];

    /**
        called on ng-init
    */
    $scope.onAttach = function () {
        list = htmlElement.querySelector('.card-list-container');
        searchInput = htmlElement.querySelector('.search');
        widget = $scope.widget;
        dataSource = widget.dataSource;
        dataSet = widget.dataSource.data || [];
        defineDataProperty();
        if (widget.dataSource.data && widget.dataSource.data.length > 0) {
            renderList();
        }
        if (widget.enableInfiniteScroll !== false) {
            list.addEventListener('scroll', event => {
                if (!$scope.childrenVisiible && !isLoading && (list.offsetHeight + list.scrollTop >= list.scrollHeight)) {
                    isLoading = true;
                    dataSource.load(widget.dataSourceFilter, dataSource.getNextPage()).then(() => isLoading = false);
                }
            });
        }
        widget.cardListInitialized = true;
    };

    function translate(fieldLabel) {
        return ScreenService.i18n(fieldLabel);
    }

    function removeCard(id, widget) {
        if (list) {
            for (var i = 0; i < list.children.length; i++) {
                if (list.children[i].key === id) {
                    list.removeChild(list.children[i]);
                    return;
                }
            }
        } else {
            var dataSource = widget.container.getWidget('alcadas').dataSource.data;
            index = dataSource.indexOf(dataSource.find(dts => dts.NRDEMANDA === id));
            dataSource = dataSource.splice(index, 1);
            widget.container.getWidget('alcadas').reload();
            return true;
        }
    }
    
    function approvesCompetency(isApproving, justification) {
        let key = widget.currentRow.NRDEMANDA;
        let operation = isApproving ? 'APROVAR' : 'REPROVAR';
        ApplicationContext.ApprovalService.approvesAlcada(widget.currentRow, operation, justification).then(responseData => {
            removeCard(key, widget);
            destroyJustificationBOX();
            let message = responseData.messages[0].message;
            if (message == 'emailNotSend') {
                MessageService.showMessageByCode("EMAIL_NOT_SEND");
            } else if(message == 'Aprovada com sucesso!') {
                MessageService.showMessageByCode("SUCCESS_APPROVED");
            } else if(message == 'Reprovada com sucesso!') {
                MessageService.showMessageByCode("SUCCESS_REJECT");
            }
            if (document.getElementsByClassName('details-action')[0]) {
                document.getElementsByClassName('details-action')[0].classList.add('hidden-actions');
            }
        }, errorResponse => {
            if(errorResponse.error === 'NO_ALCADA_WORKFLOW') {
                ApplicationContext.ApprovalService.getConsisteOrca(widget.currentRow).then(response => {
                    if(response && response.dataset && response.dataset.data && response.dataset.data.length) {
                        const data = response.dataset.data[0];

                        if(data.IDORIGEMCONS === 'CONSUMO') {
                            MessageService.showMessageByCode('NO_ALCADA_WORKFLOW_CONSUMO');
                        } else {
                            MessageService.showMessageByCode('NO_ALCADA_WORKFLOW_DESPESA', {CDDESPPREV: data.CDDESPPREV});
                        }                        
                    } else {
                        MessageService.showMessageByCode('NO_ALCADA_WORKFLOW');
                    }
                }).catch((error) => {
                    ScreenService.showMessage(error.data.error);
                });                
            } else {
                MessageService.getMessageByCode(errorResponse.error).then((message) => {
                    if(!isApproving) {
                        MessageService.showMessageByCode('FAIL_REJECT', {PARAM: message});
                    } else {
                        MessageService.showMessageByCode('FAIL_APPROVE', {PARAM: message});
                    }
                }, () => {
                    if(!isApproving) {
                        MessageService.showMessageByCode('FAIL_REJECT', {PARAM: errorResponse.error});
                    } else {
                        MessageService.showMessageByCode('FAIL_APPROVE', {PARAM: errorResponse.error});
                    }
                });
            }            
        });
    }

    function saveJustify(required) {
        let justification = document.getElementsByClassName('justification-box')[0].getElementsByTagName('textarea')[0].value;
        document.getElementsByClassName('justification-error')[0].classList.remove('show');
        if (!required || (required && justification.length > 10)) {
            widget.currentRow.justification = justification;
            approvesCompetency(!required, justification);
        } else {
            document.getElementsByClassName('justification-error')[0].classList.add('show');
            return false;
        }
        document.getElementsByTagName('body')[0].classList.remove('stop-scroll');
    }

    function cancelJustify(required) {
        document.getElementsByTagName('body')[0].classList.remove('stop-scroll');
        destroyJustificationBOX();
    }

    function destroyJustificationBOX() {
        let justificationBox = document.getElementsByClassName('justification-box')[0];
        justificationBox.classList.remove('open');
        setTimeout(function () {
            justificationBox = justificationBox.parentElement;
            justificationBox.remove(justificationBox);
        }, 400);
    }

    function renderJustificationBOX(required) {
        let overlay = document.createElement('div');
        overlay.classList.add('justification-box-overlay');

        let justificationBox = document.createElement('div');
        justificationBox.classList.add('justification-box');

        let textJustificationLabel = document.createElement('label');
        let requiredMark = !required ? "*" : "";

        let textJustification = document.createElement('textarea');
        let justificationError = document.createElement('span');
        justificationError.classList.add('justification-error');

        let actionsArea = document.createElement('div');
        actionsArea.classList.add('actions-area');

        let saveButton = document.createElement('button');
        

        let cancelButton = document.createElement('button');
        
        MessageService.getMessageByCode('JUSTIFY').then(function(message) {
            textJustificationLabel.textContent = message + requiredMark + ':';
        });
        
        MessageService.getMessageByCode('JUSTIFY_LENGTH').then(function(message) {
            justificationError.textContent = message;
        });
        
        MessageService.getMessageByCode('SAVE').then(function(message) {
            saveButton.textContent = message;
            saveButton.addEventListener('click', function () {
                saveJustify(required);
            }, false);
        });
        
        MessageService.getMessageByCode('CANCEL').then(function(message) {
            cancelButton.textContent = message;
            cancelButton.addEventListener('click', function () {
                cancelJustify();
            }, false);
        });

        actionsArea.appendChild(saveButton);
        actionsArea.appendChild(cancelButton);

        justificationBox.appendChild(textJustificationLabel);
        justificationBox.appendChild(textJustification);
        justificationBox.appendChild(justificationError);
        justificationBox.appendChild(actionsArea);

        overlay.append(justificationBox);

        return overlay;
    }

    function showJustificationBox(required) {

        let footer = document.getElementsByClassName('footer-container')[0].parentNode;
        footer.append(renderJustificationBOX(required));

        setTimeout(function () {
            let justificationBox = document.getElementsByClassName('justification-box')[0];
            justificationBox.classList.add('open');
        }, 100);
    }

    function startProcess(curAlcada, isApproving) {
        widget = widget ? widget : $scope.widget;
        widget.currentRow = widget.currentRow || widget.currentRow;
        widget.currentRow = Object.assign(widget.currentRow, curAlcada);
        widget.currentRow.isApproving = isApproving;

        let status = isApproving ? 'REALLY_APPROVE' : 'REALLY_REJECT';
        let required = isApproving ? false : true;
    
        MessageService.confirmMessageByCode(status, null, {
            type: 'question',
            success: () => {
                if(!isApproving) {
                    showJustificationBox(required);
                } else {
                    if(curAlcada.IDEXIGJUSTIFICATIVA === 'S') {
                        showJustificationBox(required);
                    } else {
                        approvesCompetency(isApproving, null);
                    }
                }
            },
            function() {
                let cardListContent = document.getElementsByTagName('body')[0];
                cardListContent.classList.add('stop-scroll');
                cardListContent.classList.remove('stop-scroll'); 
            }
        });



    }
    // $scope.onAttach = function() {
    $scope.startProcessByDetais = function (curAlcada, isApproving, widget) {
        isApproving = isApproving === "approved" ? true : false;
        startProcess(curAlcada, isApproving);
    };

    function startApprovedProcess(curDataRow) {
        startProcess(curDataRow, true);
    }

    function startRepprovedProcess(curDataRow) {
        startProcess(curDataRow, false);
    }

    function createChildElement(curDataRow) {

        let newCard = document.createElement('div');

        newCard.key = curDataRow.NRDEMANDA;
        newCard.classList.add('card-content');
        
        let firtsSection = null;
        let secondSection = null;
        let lastSection = null;
        
        for (var i = 0; i < fieldsToDisplayOnCard.length; i++) {
            let fieldTextContent = curDataRow[fieldsToDisplayOnCard[i].id] ? curDataRow[fieldsToDisplayOnCard[i].id] : '';
            let fieldLabel = "";
            if(fieldsToDisplayOnCard[i].id === "NMALCADA" || fieldsToDisplayOnCard[i].id === "NMFILIAL" || fieldsToDisplayOnCard[i].id === "IDCLASSIFIC")
                fieldLabel = ScreenService.i18n(fieldsToDisplayOnCard[i].label) + ":";
            else 
                fieldLabel = ScreenService.i18n(fieldsToDisplayOnCard[i].label)
            if (i === 0 && !firtsSection) {
                firtsSection = document.createElement('div');
                firtsSection.classList.add('firtsSection');
                newCard.appendChild(firtsSection);
            } else if (i === 2 && !secondSection) {
                secondSection = document.createElement('div');
                secondSection.classList.add('secondSection');
                newCard.appendChild(secondSection);
            } else if (i === 5 && !lastSection) {
                lastSection = document.createElement('div');
                lastSection.classList.add('lastSection');
                newCard.appendChild(lastSection);
            }

            let currentSection;
            if (i >= 0 && i <= 1) {
                currentSection = firtsSection; 
            } else if (i >= 2 && i <= 4) {
                currentSection = secondSection;
            } else if (i >= 5) {
                currentSection = lastSection;
            }

            let cardProperty = document.createElement('span');
            cardProperty.innerHTML = '<b>' + ScreenService.i18n(fieldLabel) + ' ' + '</b>' + fieldTextContent;
            currentSection.appendChild(cardProperty);
        }

        let actionsDiv = document.createElement('div');


        let fieldList = widget.fields;
        for (var i = 0; i < fieldList.length; i++) {
            var curAction = fieldList[i];
            if (!!curAction.isVisible) {
                let actionButton = document.createElement('button');
                if (curAction.isApproving === "approved") {
                    actionButton.addEventListener('click', function (event) {
                        event.stopPropagation();
                        startApprovedProcess(curDataRow);
                    }, false);
                } else {
                    actionButton.addEventListener('click', function (event) {
                        event.stopPropagation();
                        startRepprovedProcess(curDataRow);
                    }, false);
                }

                let actionButtonIcon = document.createElement('i');
                actionButtonIcon.classList.add('fas');
                actionButtonIcon.classList.add('fa-' + curAction.label);

                actionButton.appendChild(actionButtonIcon);
                actionsDiv.appendChild(actionButton);
            }
        }

        if (actionsDiv.childNodes.length > 0) {
            actionsDiv.classList.add('actions-container');
        }


        newCard.appendChild(actionsDiv);
        let viewDetailsAction = document.createElement('i');
        viewDetailsAction.className = 'fas fa-ellipsis-h card-details-action';
        viewDetailsAction.addEventListener('click', openChildren);

        let explandDetailsAction = document.createElement('i');
        explandDetailsAction.className = 'fas fa-chevron-down card-details-expander';
        explandDetailsAction.addEventListener('click', expandCard);

        newCard.appendChild(viewDetailsAction);
        newCard.appendChild(explandDetailsAction);
        return newCard;
    }

    let zhLoader = document.querySelector('div[data-zh-loader]');

    let expandCard = event => {
        let classList = event.target.parentElement.classList;
        if(classList.value.indexOf('expand-card') > 0){
            classList.remove('expand-card');
        }else{
            classList.add('expand-card');
        }
        
    };

    function setDataSourceFilter (detailsWidget, currentRow){
        detailsWidget.dataSourceFilter = [];
        let filter = ['CDFILLANC','NRDEMANDA','CDFILDEST'];
        filter.forEach(f => {
            detailsWidget.dataSourceFilter.push({
                "name": f,
                "operator": "=",
                "value": currentRow[f]
            });
        });
        detailsWidget.reload();
    }

    let openChildren = event => {
        let elementData = dataSet.find(dataElement => dataElement.NRDEMANDA === event.currentTarget.parentElement.key);
        if (widget.widgets && widget.widgets.length > 0) {
            zhLoader.classList.add('force-display-none');
            widget.currentRow = Object.assign({}, elementData);
            let detailsWidget = widget.widgets.find(w=> w.name === 'details').widgets.find(w=> w.name === 'description');
            if(detailsWidget){
                setDataSourceFilter(detailsWidget, widget.currentRow);
                detailsWidget.getField('approvalChart').isVisible = widget.currentRow.IDTPALCADA !== 'CATALOGO'; 
            }
            $scope.childrenVisiible = true;
            $scope.$root.activityStack = $scope.$root.activityStack || [];
            $scope.$root.activityStack.push(() => {
                $scope.childrenVisiible = false;
                widget.activate();
            });
            widget.widgets[0].activate();
            $scope.$apply();
        }
    };

    let openChildrenFromNotification = (event) => {
        console.log(event);
        let elementData = event.detail;
        let widget = $scope.widget;
        if (widget.widgets && widget.widgets.length > 0) {
            zhLoader.classList.add('force-display-none');
            widget.currentRow = Object.assign({}, elementData);
            let detailsWidget = widget.widgets.find(w=> w.name === 'details').widgets.find(w=> w.name === 'description');
            if(detailsWidget){
                setDataSourceFilter(detailsWidget, widget.currentRow);
            }
            $scope.childrenVisiible = true;
            $scope.$root.activityStack = $scope.$root.activityStack || [];
            $scope.$root.activityStack.push(() => {
                $scope.childrenVisiible = false;
                widget.activate();
            });
            widget.widgets[0].activate();
            // $scope.$apply();
        }
    };

    /**
     * @param {array} elementsToConcat an array of objects to render, if this param is falsy the entire dataSet will be rendered.
     */
    function renderList(elementsToConcat) {
        if (list) {
            if (!elementsToConcat) {
                list.innerHTML = '';
            }
            let elementsToRender = elementsToConcat || dataSet;

            if (dataSet.length === 0) {
                let emptyListWarning = document.createElement('span');
                MessageService.getMessageByCode('NO_DATA').then(function(message) {
                    emptyListWarning.innerHTML = message;
                    emptyListWarning.classList.add('empty-list-warning');
                    list.appendChild(emptyListWarning);
                });
            } else {
                elementsToRender.forEach(data => {
                    list.appendChild(createChildElement(data));
                });
            }
        } else {
            console.log('%cPANIC: an attempt to render elements before initial attach was made', 'font-size: 15px;background-color:red;');
        }
    }


    function defineDataProperty() {
        Object.defineProperty(dataSource, "data", {
            set: newValue => {
                if (dataSource.getCurrentPage() === 1) {
                    // render all values
                    dataSet = newValue;
                    renderList();
                } else {
                    // render only new values
                    let newValuesBegining = dataSet.length;
                    renderList(newValue.slice(newValuesBegining));
                    dataSet = newValue;
                }
            },
            get: () => dataSet
        });
    }

    document.addEventListener('notificationopen', openChildrenFromNotification);
    
}